% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataRetrieval-package.R
\docType{data}
\name{stateCd}
\alias{stateCd}
\title{US State Code Lookup Table}
\value{
stateCd data frame.

\tabular{lll}{
Name \tab Type \tab Description\cr
STATE \tab character \tab FIPS State Code  \cr
STUSAB \tab character \tab Official United States Postal Service (USPS) Code \cr
STATE_NAME \tab character \tab State Name \cr
STATENS \tab character \tab  Geographic Names Information System Identifier (GNISID)\cr
}
}
\description{
Classic lookup table for states. Has been replaced in functions with
\code{check_waterdata_sample_params("states")}.
}
\examples{
head(stateCd)
}
\keyword{USGS}
\keyword{stateCd}
