% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ogc_data.R
\name{check_arguments_api}
\alias{check_arguments_api}
\title{Check other arguments}
\usage{
check_arguments_api(bbox, skipGeometry)
}
\arguments{
\item{bbox}{Only features that have a geometry that intersects the bounding
box are selected.The bounding box is provided as four or six numbers, depending
on whether the coordinate reference system includes a vertical axis (height or
depth). Coordinates are assumed to be in crs 4326. The expected format is a numeric
vector structured: c(xmin,ymin,xmax,ymax).
Another way to think of it is c(Western-most longitude,
Southern-most latitude, Eastern-most longitude, Northern-most longitude).}

\item{skipGeometry}{This parameter can be used to skip response geometries for
each feature. The returning object will be a data frame with no spatial
information. The default \code{NA} will not specify the argument in the request.}
}
\description{
Additional functions to check types and create parameter descriptions.
}
\keyword{internal}
