% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setAccess.R
\name{setAccess}
\alias{setAccess}
\title{Set data endpoint}
\usage{
setAccess(access = "public")
}
\arguments{
\item{access}{code for data access. Options are: "public","internal","cooperator", or "USGS".
\itemize{
\item{"internal" represents Access=3 ...for a single water science center}
\item{"USGS" represents Access=2 ...for all water science centers}
\item{"cooperator" represents Access=1}
\item{"public" represents Access=0, public access}
}}
}
\description{
access Indicate which dataRetrieval access code
you want to use options: \code{c('public','internal')}
}
\examples{
\donttest{
setAccess("internal")

setAccess("public")
}

}
\author{
Luke Winslow, Jordan S Read
}
