% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findNLDI.R
\name{get_nldi_sources}
\alias{get_nldi_sources}
\title{Get current NLDI offerings}
\usage{
get_nldi_sources(url = getOption("dataRetrieval.nldi_base"))
}
\arguments{
\item{url}{URL for NLDI sources. Default is supplied by package environment.}
}
\value{
data.frame
}
\description{
Used to query the current resources available through the NLDI
}
\examples{
\dontshow{if (is_dataRetrieval_user()) withAutoprint(\{ # examplesIf}
\donttest{
get_nldi_sources()
}
\dontshow{\}) # examplesIf}
}
\keyword{nldi}
