% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataRetrieval-package.R
\docType{data}
\name{parameterCdFile}
\alias{parameterCdFile}
\title{List of USGS parameter codes}
\value{
parameterData data frame with information about USGS parameters.

\tabular{lll}{
Name \tab Type \tab Description\cr
parameter_cd \tab character \tab 5-digit USGS parameter code \cr
parameter_group_nm \tab character \tab USGS parameter group name\cr
parameter_nm \tab character \tab USGS parameter name\cr
casrn \tab character \tab Chemical Abstracts Service (CAS) Registry Number\cr
srsname \tab character \tab Substance Registry Services Name\cr
parameter_units \tab character \tab Parameter units\cr
}
}
\description{
Complete list of USGS parameter codes as of Oct. 24, 2024.
}
\examples{
head(parameterCdFile[, 1:2])
}
