% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AAA.R
\name{is_dataRetrieval_user}
\alias{is_dataRetrieval_user}
\title{Is this a dataRetrieval user}
\usage{
is_dataRetrieval_user()
}
\description{
Reveals if this is a user or not
}
\examples{
is_dataRetrieval_user()
}
