% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkWQPdates.R
\name{checkWQPdates}
\alias{checkWQPdates}
\title{Date Check for Water Quality Portal}
\usage{
checkWQPdates(values)
}
\arguments{
\item{values}{named list with arguments to send to the Water Quality Portal}
}
\value{
values named list with corrected arguments to send to the Water Quality Portal
}
\description{
Checks date format for inputs to the Water Quality Portal. Used in \code{readWQPqw}
and \code{readWQPdata}.
}
\examples{
values <- list(
  startDateLo = "01-01-2002",
  characteristicName = "Phosphorous",
  endDate = as.Date("2014-01-01")
)
values <- checkWQPdates(values)
}
