% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ogc_documentation.R
\name{get_ogc_params}
\alias{get_ogc_params}
\title{Get parameter descriptions}
\usage{
get_ogc_params(service)
}
\arguments{
\item{service}{Character, can be any of the USGS Waterdata API endpoints or collections.}
}
\value{
list
}
\description{
This function returns a list of properties available from an endpoint. When
available, it will also contain a description.
}
\examples{
\dontshow{if (is_dataRetrieval_user()) withAutoprint(\{ # examplesIf}

\donttest{
ml <- dataRetrieval::get_ogc_params("monitoring-locations")
ml$national_aquifer_code
}
\dontshow{\}) # examplesIf}
}
