% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataRetrieval-package.R
\docType{package}
\name{dataRetrieval}
\alias{dataRetrieval}
\alias{dataRetrieval-package}
\title{Retrieval functions for USGS and EPA data}
\description{
\tabular{ll}{
Package: \tab dataRetrieval\cr
Type: \tab Package\cr
License: \tab Unlimited for this package, dependencies have more restrictive licensing.\cr
Copyright: \tab This software is in the public domain because it contains materials
that originally came from the United States Geological Survey, an agency of
the United States Department of Interior. \cr
LazyLoad: \tab yes\cr
}
}
\details{
Retrieval functions for USGS and EPA hydrologic and water quality data.

Please see \url{https://doi-usgs.github.io/dataRetrieval/} for more information.
}
\seealso{
Useful links:
\itemize{
  \item Report bugs at \url{https://github.com/DOI-USGS/dataRetrieval/issues}
}

}
\author{
Laura De Cicco \email{ldecicco@usgs.gov}
}
\keyword{internal}
