% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataRetrieval-package.R
\docType{data}
\name{countyCd}
\alias{countyCd}
\title{US County Code Lookup Table}
\value{
countyCd data frame.

\tabular{lll}{
Name \tab Type \tab Description\cr
STUSAB \tab character \tab State abbreviation \cr
STATE \tab character \tab two-digit ANSI code  \cr
COUNTY \tab character \tab three-digit county code \cr
COUNTY_NAME \tab character \tab County full name \cr
COUNTY_ID \tab character \tab County id \cr
}
}
\description{
Classic lookup table for counties. Has been replaced in functions with
\code{check_waterdata_sample_params("counties")}.
}
\examples{
head(countyCd)
}
\keyword{USGS}
\keyword{countyCd}
