% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readNWISdata.R
\name{readNWISdots}
\alias{readNWISdots}
\title{Format and organize NWIS arguments that are passed in as \code{...}.}
\usage{
readNWISdots(...)
}
\description{
Format and organize NWIS arguments that are passed in as \code{...}.
}
\keyword{internal}
