% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataRetrieval-package.R
\docType{data}
\name{pCodeToName}
\alias{pCodeToName}
\title{Data to convert USGS parameter code to characteristic names}
\value{
pCodeToName data frame with information about USGS parameters and how they
relate to characteristic names (useful for WQP requests).

\tabular{lll}{
Name \tab Type \tab Description\cr
parm_cd \tab character \tab 5-digit USGS parameter code \cr
description \tab character \tab Parameter description\cr
characteristicname \tab character \tab Characteristic Name \cr
measureunitcode \tab character \tab Parameter units\cr
resultsamplefraction \tab character \tab Result sample fraction text\cr
resulttemperaturebasis \tab character \tab Temperature basis information\cr
resultstatisticalbasis \tab character \tab Statistical basis\cr
resulttimebasis \tab character \tab Time basis\cr
resultweightbasis \tab character \tab Weight basis\cr
resultparticlesizebasis \tab character \tab Particle size basis\cr
last_rev_dt \tab character \tab Latest revision of information\cr
}
}
\description{
Data pulled from Water Quality Portal on December 20, 2021. The data was pulled from
\url{https://www.waterqualitydata.us/Codes/public_srsnames/?mimeType=csv}.
}
\examples{
head(pCodeToName[, 1:2])
}
\keyword{internal}
