% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_waterdata_samples.R
\name{read_USGS_samples}
\alias{read_USGS_samples}
\title{Deprecated: Use \code{read_waterdata_samples} instead}
\usage{
read_USGS_samples(...)
}
\arguments{
\item{...}{Arguments passed to \code{\link{read_waterdata_samples}}.}
}
\value{
data frame returned from web service call.
}
\description{
This function has been renamed to \code{\link{read_waterdata_samples}}.
}
\keyword{internal}
