% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readNWISpCode.R
\name{readNWISpCode}
\alias{readNWISpCode}
\title{USGS Parameter Data Retrieval}
\usage{
readNWISpCode(parameterCd)
}
\arguments{
\item{parameterCd}{character of USGS parameter codes (or multiple parameter codes).
These are 5 digit number codes. To get a complete list of all current parameter
codes in the USGS, use "all" as the input.}
}
\value{
parameterData data frame with the following information:
\tabular{lll}{
Name \tab Type \tab Description\cr
parameter_cd \tab character \tab 5-digit USGS parameter code \cr
parameter_group_nm \tab character \tab USGS parameter group name\cr
parameter_nm \tab character \tab USGS parameter name\cr
casrn \tab character \tab Chemical Abstracts Service (CAS) Registry Number\cr
srsname \tab character \tab Substance Registry Services Name\cr
parameter_units \tab character \tab Parameter units\cr
}
}
\description{
Imports data from NWIS about measured parameter based on user-supplied parameter code or codes.
}
\seealso{
\code{\link[=importRDB1]{importRDB1()}}
}
\keyword{USGS}
\keyword{data}
\keyword{import}
\keyword{service}
\keyword{web}
