% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readNWISdata.R
\name{convertLists}
\alias{convertLists}
\title{convert variables in dots to usable format}
\usage{
convertLists(...)
}
\description{
convert variables in dots to usable format
}
\keyword{internal}
