% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readWQPdots.R
\name{readWQPdots}
\alias{readWQPdots}
\title{Format and organize WQP arguments that are passed in as \code{...}.}
\usage{
readWQPdots(..., legacy = TRUE)
}
\description{
Format and organize WQP arguments that are passed in as \code{...}.
}
\keyword{internal}
