% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ogc_documentation.R
\name{check_OGC_requests}
\alias{check_OGC_requests}
\title{Check OGC requests}
\usage{
check_OGC_requests(endpoint = "daily", type = "queryables")
}
\arguments{
\item{endpoint}{Character, can be any existing collection}

\item{type}{Character, can be "queryables", "schema"}
}
\value{
list
}
\description{
Check OGC requests
}
\examples{
\dontshow{if (is_dataRetrieval_user()) withAutoprint(\{ # examplesIf}

\donttest{

dv_queryables <- check_OGC_requests(endpoint = "daily",
                                type = "queryables")
dv_schema <- check_OGC_requests(endpoint = "daily",
                            type = "schema")
ts_meta_queryables <- check_OGC_requests(endpoint = "time-series-metadata",
                                type = "queryables")
ts_meta_schema <- check_OGC_requests(endpoint = "time-series-metadata",
                                type = "schema")
}
\dontshow{\}) # examplesIf}
}
\keyword{internal}
