% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getWebServiceData.R
\name{has_internet_2}
\alias{has_internet_2}
\title{has_internet_2}
\usage{
has_internet_2(obs_url)
}
\arguments{
\item{obs_url}{character obs_url to check}
}
\description{
Function to check for internet even if the user
is behind a proxy
}
\details{
If this is giving you problems, override the false negative:
Sys.setenv("OVERRIDE_INTERNET_TEST" = TRUE)
}
\keyword{internal}
