% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getWebServiceData.R
\name{getQuerySummary}
\alias{getQuerySummary}
\title{getting header information from a WQP query}
\usage{
getQuerySummary(url)
}
\arguments{
\item{url}{the query url}
}
\description{
getting header information from a WQP query
}
